%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%Info%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Description: A simple one dimensional finite difference scheme.         %
%                                                                         %
% Adapted from a snippet of code from the following                       %
% publication (source):                                                   %
%                                                                         %
% Nathan J. Dawson and Mark G. Kuzyk, "Polynomial potentials determined   % 
% from the energy spectrum and transition dipole moments that give the    %
% largest hyperpolarizabilities," J. Opt. Soc. Am. B, 33, E83-E92 (2016). %
%                                                                         %
% Author: N. J. Dawson                                                    %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function Schrodinger1D

%%initial parameters
hbar = 1; e = 1; m = 1; epsilon0 = 1/(4*pi); % set constants for atomic units (a. u.)
nmodes = 4; %number of eigenvalues and eigenvectors to calculate
Vdispamp = 20; %scale amplitude of wavefunction for graphical viewing
Vdisp2amp = 40^2; %scale amplitude of probability density for graphical viewing

%%set up space to calculate
xlft = -4; %left boundary location in Bohr
xrght = 4; %right boundary location in Bohr
N = 10001; %number of steps to calculate
x = linspace(xlft,xrght,N)'; %create the position vector between boundaries

%%setup an example potential
omega = 1; %angular frequency of QHO potential in a. u.
U = 0.5*(omega^2)*(x.^2); %potential energy function (QHO)

%%calculate using finite difference scheme
dx = x(2) - x(1); % displacement step size in bohr
el = ones(N,1); %create vector to set up tridiagonal matrix
Lapl = spdiags([el -2*el el],[-1 0 1],N,N)/dx^2; %create laplacian matrix
H = -1/2*(hbar^2/m)*Lapl + spdiags(U,0,N,N); %create Hamiltonian by adding the potential to the laplacian diagonals
[Veig,Eeig] = eigs(H,nmodes,'sm'); %find eigenvalues 'Eeig' and normalized eigenvectors 'Veig' of sparse matrix
[Eeig,ind] = sort(diag(Eeig)); %sort the eigenvalues from smallest to largest
Veig = Veig(:,ind); %sort the eigenvectors based on the eigenvalue sorting
E = Eeig(1:nmodes,1)'; %create a list of eigenvalues
V = Veig(:,1:nmodes); %create an array of eigenvectors

%%plot wavefunction & probability density figures with potential energy
dispV = ones(N,1)*E(1,1:nmodes) + Vdispamp*sign(Veig(round(N/8)))*Veig(:,1:nmodes); %set up wavefunctions to easily display by energy offset and increase amplitude
dispV2 = ones(N,1)*E(1,1:nmodes) + Vdisp2amp*Veig(:,1:nmodes).^2; %set up probability distribution to easily display by energy offset and increase amplitude
figure(1)
plot(x,U,x,dispV(:,1),x,dispV(:,2),x,dispV(:,3),x,dispV(:,4)) %plot potential energy and first four wavefunctions
figure(2)
plot(x,U,x,dispV2(:,1),x,dispV2(:,2),x,dispV2(:,3),x,dispV2(:,4)) %plot potential energy and first four probability density

%%display energy eigenvalues
Eeig

end

