#############################Info##############################################
# Description: A simple one dimensional finite difference scheme.             #
#                                                                             #
# Adapted from a snippet of code from the following                           #
# publication (source):                                                       #
#                                                                             #
# Nathan J. Dawson and Mark G. Kuzyk, “Polynomial potentials determined       # 
# from the energy spectrum and transition dipole moments that give the        #
# largest hyperpolarizabilities,” J. Opt. Soc. Am. B, 33, E83-E92 (2016).     #
#                                                                             #
# Author: N. J. Dawson                                                        #
###############################################################################

import matplotlib.pyplot as plt
import numpy as np
from scipy.sparse import spdiags
from scipy.sparse.linalg import eigsh
import math

##initial parameters
hbar = 1; e = 1; m = 1; epsilon0 = 1/(4*math.pi); # set constants for atomic units (a. u.)
nmodes = 4; #number of eigenvalues and eigenvectors to calculate
Vdispamp = 20; #scale amplitude of wavefunction for graphical viewing
Vdisp2amp = 40**2; #scale amplitude of probability density for graphical viewing

##set up space to calculate
xlft = -4; #left boundary location in Bohr
xrght = 4; #right boundary location in Bohr
N = 10001; #number of steps to calculate
x = np.linspace(xlft, xrght, N, endpoint=True, retstep=False); #create the position vector between boundaries

##setup an example potential
omega = 1; #angular frequency of QHO potential in a. u.
U = 0.5*(omega**2)*(x**2); #potential energy function (QHO)

##calculate using finite difference scheme
dx = x[1] - x[0]; #displacement step size in bohr
el = np.ones(N); #create vector to set up tridiagonal matrix
H = spdiags([-0.5*(hbar**2/m)*el/(dx**2),(hbar**2/m)*el/(dx**2) + U,-0.5*(hbar**2/m)*el/(dx**2)],[-1,0,1],N,N); #create sparse Hamiltonian matrix
Eeig, Veig = eigsh(H,nmodes,sigma=None,which='SM'); #find smallest eigenvalues 'Eeig' and normalized eigenvectors 'Veig' of sparse matrix sorted by smallest first

dispV = Eeig[0:nmodes] + Vdispamp*np.sign(Veig[round(N/8),0:nmodes])*Veig[:,0:nmodes]; #set up wavefunctions to easily display by energy offset and increase amplitude
dispV2 = Eeig[0:nmodes] + Vdisp2amp*Veig[:,0:nmodes]**2; #set up probability distribution to easily display by energy offset and increase amplitude

##plot wavefunction & probability density figures with potential energy
fig = plt.figure(1)
plt.plot(x,U,x,dispV[:,0],x,dispV[:,1],x,dispV[:,2],x,dispV[:,3]) #plot potential energy and first four wavefunctions
fig = plt.figure(2)
plt.plot(x,U,x,dispV2[:,0],x,dispV2[:,1],x,dispV2[:,2],x,dispV2[:,3]) #plot potential energy and first four probability densities
plt.show()

##display energy eigenvalues
print(Eeig)